@@ -56,6 +56,7 @@ class DataFile:
         )
         self.__variation_by_id: Dict[int, VariationByExposition] = {}
         self.__rule_by_segment_id: Dict[int, Rule] = {}
+        self.__rule_info_by_exp_id: Dict[int, DataFile.RuleInfo] = {}
         self.__feature_flag_by_id: Dict[int, FeatureFlag] = {}
         self.__experiment_ids_with_js_css_variable: Set[int] = set()
         self.__collect_indices()
@@ -84,6 +85,11 @@ class DataFile:
         """Returns dictionary of all rule stored by segment id"""
         return self.__rule_by_segment_id
 
+    @property
+    def rule_info_by_exp_id(self) -> Dict[int, "DataFile.RuleInfo"]:
+        """Returns dictionary of all rule related info stored by experiment id"""
+        return self.__rule_info_by_exp_id
+
     @property
     def variation_by_id(self) -> Dict[int, VariationByExposition]:
         """Returns dictionary of all variations stored by id"""
@@ -122,6 +128,7 @@ class DataFile:
                 has_feature_flag_variable_js_css = self.__has_feature_flag_variable_js_css(feature_flag)
                 for rule in feature_flag.rules:
                     self.__rule_by_segment_id[rule.segment_id] = rule
+                    self.__rule_info_by_exp_id[rule.experiment_id] = DataFile.RuleInfo(feature_flag, rule)
                     if has_feature_flag_variable_js_css:
                         self.__experiment_ids_with_js_css_variable.add(rule.experiment_id)
                     for variation in rule.variation_by_exposition:
@@ -146,3 +153,20 @@ class DataFile:
             f"settings:{self.__settings}"
             f"}}"
         )
+
+    class RuleInfo:
+        """Aggregates some information related to a specific rule"""
+
+        def __init__(self, feature_flag: FeatureFlag, rule: Rule) -> None:
+            self.__feature_flag = feature_flag
+            self.__rule = rule
+
+        @property
+        def feature_flag(self) -> FeatureFlag:
+            """Returns the feature flag"""
+            return self.__feature_flag
+
+        @property
+        def rule(self) -> Rule:
+            """Returns the rule"""
+            return self.__rule
