@@ -20,6 +20,8 @@ class DataType(IntEnum):
     VISITOR_VISITS = 11
     KCS_HEAT = 12
     UNIQUE_IDENTIFIER = 13
+    FORCED_EXPERIMENT_VARIATION = 14
+    FORCED_FEATURE_VARIATION = 15
 
 
 class BaseData:
