@@ -40,8 +40,11 @@ class Device(Data, DuplicationUnsafeSendableBase):
     def data_type(self) -> DataType:
         return DataType.DEVICE
 
-    def _add_query_params(self, qb: QueryBuilder) -> None:
-        qb.extend(
+    def _add_query_params(self, query_builder: QueryBuilder) -> None:
+        query_builder.extend(
             QueryParam(QueryParams.EVENT_TYPE, self.EVENT_TYPE),
             QueryParam(QueryParams.DEVICE_TYPE, self.device_type.value),
         )
+
+    def __str__(self):
+        return f"Device{{device_type:{self.__device_type}}}"
