@@ -70,6 +70,15 @@ class FeatureNotFound(NotFoundError):
         super().__init__(self.message)
 
 
+class FeatureExperimentNotFound(NotFoundError):  # //~ Derived from FeatureError in other SDKs
+    """Exception indicating that the requested experiment has not been found.
+    Check that the experiment's ID matches the provided one."""
+
+    def __init__(self, message=None) -> None:
+        self.message = "Experiment " + str(message)
+        super().__init__(self.message)
+
+
 class FeatureVariableNotFound(NotFoundError):
     """Exception indicating that the requested variable has not been found.
     Check that the variable's ID (or key) matches the one in your code."""
