@@ -14,10 +14,12 @@ from kameleoon.configuration.variation_by_exposition import VariationByExpositio
 
 from kameleoon.data import Conversion, CustomData, Data, UniqueIdentifier
 from kameleoon.data.manager.assigned_variation import AssignedVariation
+from kameleoon.data.manager.forced_experiment_variation import ForcedExperimentVariation
 from kameleoon.data.manager.visitor_manager import VisitorManager
 
 from kameleoon.exceptions import (
     FeatureVariationNotFound,
+    FeatureExperimentNotFound,
     FeatureVariableNotFound,
     FeatureEnvironmentDisabled,
     SiteCodeIsEmpty,
@@ -130,7 +132,7 @@ class KameleoonClient:
         )
         self._hybrid_manager = HybridManagerImpl(HYBRID_EXPIRATION_TIME, self._data_manager)
 
-        self._cookie_manager = CookieManager(self._data_manager, config.top_level_domain)
+        self._cookie_manager = CookieManager(self._data_manager, self._visitor_manager, config.top_level_domain)
         atsf = AccessTokenSourceFactory(config.client_id, config.client_secret)
         self._network_manager = self._network_manager_factory.create(
             self.site_code,
@@ -589,9 +591,13 @@ class KameleoonClient:
             "CALL: KameleoonClient.__get_variation_info(visitor_code: %s, feature_flag: %s, track: %s)",
             visitor_code, feature_flag, track
         )
-        var_by_exp, rule = self.__calculate_variation_rule_for_feature(visitor_code, feature_flag)
+        visitor = self._visitor_manager.get_visitor(visitor_code)
+        forced_variation = visitor.get_forced_feature_variation(feature_flag.feature_key) if visitor else None
+        var_by_exp, rule = (forced_variation.var_by_exp, forced_variation.rule) if forced_variation else \
+            self.__calculate_variation_rule_for_feature(visitor_code, feature_flag)
+        if not (forced_variation and forced_variation.simulated):
+            self.__save_variation(visitor_code, rule, var_by_exp, track)
         variation_key = self.__calculate_variation_key(var_by_exp, rule, feature_flag)
-        self.__save_variation(visitor_code, rule, var_by_exp, track)
         KameleoonLogger.debug(
             "RETURN: KameleoonClient.__get_variation_info(visitor_code: %s, feature_flag: %s, track: %s)"
             " -> (variation_key: %s, variation_by_exposition: %s, rule: %s)",
@@ -942,11 +948,14 @@ class KameleoonClient:
             "Call to deprecated function `get_active_feature_list_for_visitor`. "
             "Please use `get_active_features` instead."
         )
+        visitor = self._visitor_manager.get_visitor(visitor_code)
 
         def filter_conditions(feature_flag: FeatureFlag) -> bool:
             if not feature_flag.environment_enabled:
                 return False
-            (variation, rule) = self.__calculate_variation_rule_for_feature(visitor_code, feature_flag)
+            forced_variation = visitor.get_forced_feature_variation(feature_flag.feature_key) if visitor else None
+            variation, rule = (forced_variation.var_by_exp, forced_variation.rule) if forced_variation else \
+                self.__calculate_variation_rule_for_feature(visitor_code, feature_flag)
             variation_key = self.__calculate_variation_key(variation, rule, feature_flag)
             return variation_key != configuration.Variation.Type.OFF.value
 
@@ -987,11 +996,14 @@ class KameleoonClient:
         )
         KameleoonLogger.info("CALL: KameleoonClient.get_active_features(visitor_code: %s)", visitor_code)
         validate_visitor_code(visitor_code)
+        visitor = self._visitor_manager.get_visitor(visitor_code)
         map_active_features: Dict[str, Variation] = {}
         for feature_flag in self._data_manager.data_file.feature_flags.values():
             if not feature_flag.environment_enabled:
                 continue
-            (var_by_exp, rule) = self.__calculate_variation_rule_for_feature(visitor_code, feature_flag)
+            forced_variation = visitor.get_forced_feature_variation(feature_flag.feature_key) if visitor else None
+            var_by_exp, rule = (forced_variation.var_by_exp, forced_variation.rule) if forced_variation else \
+                self.__calculate_variation_rule_for_feature(visitor_code, feature_flag)
             variation_key = self.__calculate_variation_key(var_by_exp, rule, feature_flag)
             if variation_key == configuration.Variation.Type.OFF.value:
                 continue
@@ -1303,9 +1315,13 @@ class KameleoonClient:
             "CALL: KameleoonClient.__get_feature_variation_key(visitor_code: %s, feature_key: %s)",
             visitor_code, feature_key)
         feature_flag = self._data_manager.data_file.get_feature_flag(feature_key)
-        (variation, rule) = self.__calculate_variation_rule_for_feature(visitor_code, feature_flag)
+        visitor = self._visitor_manager.get_visitor(visitor_code)
+        forced_variation = visitor.get_forced_feature_variation(feature_key) if visitor else None
+        variation, rule = (forced_variation.var_by_exp, forced_variation.rule) if forced_variation else \
+            self.__calculate_variation_rule_for_feature(visitor_code, feature_flag)
+        if not (forced_variation and forced_variation.simulated):
+            self.__save_variation(visitor_code, rule, variation)
         variation_key = self.__calculate_variation_key(variation, rule, feature_flag)
-        self.__save_variation(visitor_code, rule, variation)
         self._tracking_manager.add_visitor_code(visitor_code)
         KameleoonLogger.debug(
             "RETURN: KameleoonClient.__get_feature_variation_key(visitor_code: %s, feature_key: %s) -> "
@@ -1338,28 +1354,36 @@ class KameleoonClient:
         KameleoonLogger.debug(
             "CALL: KameleoonClient.__calculate_variation_rule_for_feature(visitor_code: %s, feature_flag: %s)",
             visitor_code, feature_flag)
+        visitor = self._visitor_manager.get_visitor(visitor_code)
+        # use mappingIdentifier instead of visitorCode if it was set up
+        code_for_hash = (
+            visitor.mapping_identifier
+            if visitor is not None and visitor.mapping_identifier is not None
+            else visitor_code
+        )
+        selected_variation = None
+        selected_rule = None
         for rule in feature_flag.rules:
+            forced_variation = visitor.get_forced_experiment_variation(rule.experiment_id) if visitor else None
+            if forced_variation and forced_variation.force_targeting:
+                # Forcing experiment variation in force-targeting mode
+                selected_variation, selected_rule = forced_variation.var_by_exp, rule
+                break
             # check if visitor is targeted for rule, else next rule
             if not self.__check_targeting(visitor_code, rule.experiment_id, rule):
                 continue
-            visitor = self._visitor_manager.get_visitor(visitor_code)
-            # use mappingIdentifier instead of visitorCode if it was set up
-            code_for_hash = (
-                visitor.mapping_identifier
-                if visitor is not None and visitor.mapping_identifier is not None
-                else visitor_code
-            )
+            if forced_variation:
+                # Forcing experiment variation in targeting-only mode
+                selected_variation, selected_rule = forced_variation.var_by_exp, rule
+                break
             # uses for rule exposition
             hash_rule = obtain_hash_double_rule(code_for_hash, rule.id_, rule.respool_time)
             KameleoonLogger.debug("Calculated hash_rule: %s for visitor_code: %s", hash_rule, code_for_hash)
             # check main expostion for rule with hashRule
             if hash_rule <= rule.exposition:
                 if rule.is_targeted_delivery:
-                    KameleoonLogger.debug(
-                        "RETURN: KameleoonClient.__calculate_variation_rule_for_feature(visitor_code: %s, "
-                        "feature_flag: %s) -> (variation: %s, rule: %s)",
-                        visitor_code, feature_flag, rule.first_variation, rule)
-                    return (rule.first_variation, rule)
+                    selected_variation, selected_rule = rule.first_variation, rule
+                    break
 
                 # uses for variation's expositions
                 hash_variation = obtain_hash_double_rule(code_for_hash, rule.experiment_id, rule.respool_time)
@@ -1368,17 +1392,14 @@ class KameleoonClient:
                 # get variation with hash_variation
                 variation = rule.get_variation(hash_variation)
                 if variation:
-                    KameleoonLogger.debug(
-                        "RETURN: KameleoonClient.__calculate_variation_rule_for_feature(visitor_code: %s, "
-                        "feature_flag: %s) -> (variation: %s, rule: %s)",
-                        visitor_code, feature_flag, variation, rule)
-                    return (variation, rule)
+                    selected_variation, selected_rule = variation, rule
+                    break
             elif rule.is_targeted_delivery:
                 break
         KameleoonLogger.debug(
             "RETURN: KameleoonClient.__calculate_variation_rule_for_feature(visitor_code: %s, feature_flag: %s) "
-            "-> (variation: None, rule: None)", visitor_code, feature_flag)
-        return None, None
+            "-> (variation: %s, rule: %s)", visitor_code, feature_flag, selected_variation, selected_rule)
+        return selected_variation, selected_rule
 
     @staticmethod
     def __calculate_variation_key(
@@ -1409,3 +1430,49 @@ class KameleoonClient:
     def _is_consent_given(self, visitor_code: str) -> bool:
         return not self._data_manager.data_file.settings.is_consent_required or \
             ((visitor := self._visitor_manager.get_visitor(visitor_code)) is not None) and visitor.legal_consent
+
+    def set_forced_variation(
+        self, visitor_code: str, experiment_id: int, variation_key: Optional[str], force_targeting=True
+    ) -> None:
+        """
+        Sets or resets a forced variation for a visitor in a specific experiment,
+        so the experiment will be evaluated to the variation for the visitor.
+
+        In order to reset the forced variation set the `variation_key` parameter to `None`.
+        If the forced variation you want to reset does not exist, the method will have no effect.
+
+        :param visitor_code: The unique visitor code identifying the visitor.
+        :type visitor_code: str
+        :param experiment_id: The identifier of the experiment you want to set/reset the forced variation for.
+        :type experiment_id: int
+        :param variation_key: The identifier of the variation you want the experiment to be evaluated to. \
+            Set to `None` to reset the forced variation.
+        :type variation_key: Optional[str]
+        :param force_targeting: If `True`, the visitor will be targeted to the experiment regardless its conditions. \
+            Otherwise, the normal targeting logic will be preserved. Optional (defaults to `True`).
+        :type force_targeting: bool
+
+        :raises VisitorCodeInvalid: The provided **visitor code** is invalid.
+        :raises FeatureExperimentNotFound: The provided **experiment id** does not exist in the feature flag.
+        :raises FeatureVariationNotFound: The provided **variation key** does not belong to the experiment.
+        """
+        KameleoonLogger.info(
+            "CALL: KameleoonClient.set_forced_variation(visitor_code: %s, experiment_id: %s, variation_key: %s, "
+            "force_targeting: %s)", visitor_code, experiment_id, variation_key, force_targeting
+        )
+        validate_visitor_code(visitor_code)
+        if variation_key is None:
+            visitor = self._visitor_manager.get_visitor(visitor_code)
+            if visitor:
+                visitor.reset_forced_experiment_variation(experiment_id)
+        else:
+            rule_info = self._data_manager.data_file.rule_info_by_exp_id.get(experiment_id)
+            if rule_info is None:
+                raise FeatureExperimentNotFound(experiment_id)
+            var_by_exp = rule_info.rule.get_variation_by_key(variation_key)
+            forced_variation = ForcedExperimentVariation(rule_info.rule, var_by_exp, force_targeting)
+            self._visitor_manager.add_data(visitor_code, forced_variation)
+        KameleoonLogger.info(
+            "RETURN: KameleoonClient.set_forced_variation(visitor_code: %s, experiment_id: %s, variation_key: %s, "
+            "force_targeting: %s)", visitor_code, experiment_id, variation_key, force_targeting
+        )
