@@ -38,6 +38,7 @@ class RemoteVisitorData:
         self.__experiments: Optional[Dict[int, AssignedVariation]] = None
         self.__conversions: Optional[List[Conversion]] = None
         self.__previous_visitor_visits: Optional[VisitorVisits] = None
+        self.__visitor_code: Optional[str] = None
         current_visit = json.get("currentVisit")
         if current_visit is not None:
             self.__parse_visit(current_visit)
@@ -100,7 +101,14 @@ class RemoteVisitorData:
             for conversion in self.__conversions:
                 conversion.mark_as_sent()
 
+    @property
+    def visitor_code(self) -> Optional[str]:
+        """Returns the `visitorCode` from the most recent visit"""
+        return self.__visitor_code
+
     def __parse_visit(self, json_visit: Dict[str, Any]) -> None:
+        if (self.__visitor_code is None) and isinstance(visitor_code := json_visit.get("visitorCode"), str):
+            self.__visitor_code = visitor_code
         custom_data_events = json_visit.get("customDataEvents")
         if custom_data_events:
             self.__parse_custom_data(custom_data_events)
